using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Linq;
using System.Threading.Tasks;
using System.Windows.Forms;
using DevExpress.XtraBars;
using SystemX.Product.ALIS.UI.View;
using static SystemX.Product.ALIS.UI.View.ViewCfg;
using SystemX.Net.Platform.Common.Util;
using DataBaseConnection.Control;
using System.Data.SqlClient;
using SystemX.Product.ALIS.UI.Subs;
using System.Diagnostics;
using static SystemX.Net.Platform.Common.Util.LogMessage;
using SystemX.Product.ALIS.Interface;
using System.Net;
using static SystemX.Product.CP.TRA.MainForm;
using SystemX.Product.TRA.UIControl;

namespace SystemX.Product.CP.TRA
{
    public partial class MainForm
    {
        //IChildControl
        public void SetOverview()
        {
            SelectOverviewTab();
        }
        public void SetDetailOverview()
        {
            SelectDetailOverviewTab();
        }

        public void SetHistoryView()
        {
            SetHostHistoryView();
        }

        public void SetSummaryView()
        {
            SetHostTrendView();
        }

        public void CallDetailOverView()
        {
            SetDetailOerviewFindData(SelectedData, barToggleSwitchItemOverviewMerge.Checked);
        }

        public void AnnounceContentsType(eContents GetType)
        {
            SelectedContents = GetType;
        }

        public bool GetOverviewMergeState()
        {
            return barToggleSwitchItemOverviewMerge.Checked;
        }

        public SelectedDataCollection GetSelectDataOverView(int nDataValue = int.MaxValue)
        {
            SelectedDataCollection MakeSelectData = (SelectedDataCollection)SelectedData.Clone();

            DateTime dtStart = DateTime.Parse(barEditItemDateStart.EditValue.ToString());
            DateTime dtEnd = DateTime.Parse(barEditItemDateEnd.EditValue.ToString());

            if (nDataValue != int.MaxValue)
                dtStart = dtStart.AddDays(-nDataValue);

            dtEnd = dtEnd.AddHours(23.99999);

            MakeSelectData.SearchRangeStart = dtStart;
            MakeSelectData.SearchRangeEnd = dtEnd;

            return MakeSelectData;
        }

        public void SetNgHistView(eContents GetCallType)
        {
            SetNgHistoryView(GetCallType);
        }
    }
}